@@ -14,7 +14,6 @@
 #ifndef _LG_DICT_STRUCTURES_H_
 #define _LG_DICT_STRUCTURES_H_
 
-#include "link-grammar/link-features.h"
 #include "link-includes.h"
 
 LINK_BEGIN_DECLS
@@ -22,7 +21,6 @@ LINK_BEGIN_DECLS
 /* Forward decls */
 typedef struct Dict_node_struct Dict_node;
 typedef struct Exp_struct Exp;
-typedef struct E_list_struct E_list;
 typedef struct Word_file_struct Word_file;
 typedef struct condesc_struct condesc_t;
 
@@ -36,39 +34,77 @@ typedef enum
 	CONNECTOR_type
 } Exp_type;
 
+static const int cost_max_dec_places = 3;
+static const double cost_epsilon = 1E-5;
+
+#define EXPTAG_SZ 100 /* Initial size for the Exptag array. */
+typedef enum { Exptag_none=0, Exptag_dialect, Exptag_macro } Exptag_type;
+
 /**
- * The E_list and Exp structures defined below comprise the expression
- * trees that are stored in the dictionary.  The expression has a type
- * (OR_type, AND_type or CONNECTOR_type).  If it is not a terminal it
- * has a list (an E_list) of children. Else "condesc" is the connector
- * descriptor, when "dir" indicates the connector direction.
+ * The Exp structure defined below comprises the expression trees that are
+ * stored in the dictionary. The expression has a type (OR_type, AND_type
+ * or CONNECTOR_type). If it is not a terminal "operand_first" points to its
+ * list of operands (when each of them points to the next one through
+ * "operand_next"). Else "condesc" is the connector descriptor, when "dir"
+ * indicates the connector direction.
  */
 struct Exp_struct
 {
-	Exp * next;    /* Used only for memory management, for freeing */
-	Exp_type type; /* One of three types: AND, OR, or connector. */
-	char dir;      /* The connector connects to: '-': the left; '+': the right */
-	bool multi;    /* TRUE if a multi-connector (for connector)  */
-	union {
-		E_list * l;           /* Only needed for non-terminals */
-		condesc_t * condesc;  /* Only needed if it's a connector */
-	} u;
-	double cost;   /* The cost of using this expression.
-	                  Only used for non-terminals */
+	Exp_type type:8;      /* One of three types: AND, OR, or connector. */
+	unsigned int unused:24;
+	union
+	{
+		struct /* For non-terminals. */
+		{
+			Exptag_type tag_type:8; /* tag_id namespace. */
+			unsigned int tag_id:24; /* Index in tag_type namespace. */
+		};
+		struct /* For connectors */
+		{
+			bool multi; /* TRUE if a multi-connector. */
+			char dir;   /* Connects to the left ('-') or right ('+'). */
+			unsigned char farthest_word; /* See Connector_struct. */
+		};
+	};
+	float cost;            /* The cost of using this expression. */
+	union
+	{
+		Exp *operand_first; /* First operand (for non-terminals). */
+		condesc_t *condesc; /* Only needed if it's a connector. */
+	};
+	Exp *operand_next;     /* Next same-level operand. */
 };
 
-struct E_list_struct
+/* List of words in a dictionary category. */
+typedef struct
 {
-	E_list * next;
-	Exp * e;
-};
+	unsigned int num_words;
+	const char* name;
+	Exp *exp;
+	char const ** word;
+} Category;
+
+/* List of disjuncts categories and their costs. */
+typedef struct
+{
+	unsigned int num;    /* Index in the Category array. */
+	float cost;          /* Corresponding disjunct cost. */
+} Category_cost;
+
+bool cost_eq(double cost1, double cost2);
+const char *cost_stringify(double cost);
 
 /* API to access the above structure. */
 static inline Exp_type lg_exp_get_type(const Exp* exp) { return exp->type; }
 static inline char lg_exp_get_dir(const Exp* exp) { return exp->dir; }
 static inline bool lg_exp_get_multi(const Exp* exp) { return exp->multi; }
-const char* lg_exp_get_string(const Exp*);
 static inline double lg_exp_get_cost(const Exp* exp) { return exp->cost; }
+static inline Exp* lg_exp_operand_first(Exp* exp) { return exp->operand_first; }
+static inline Exp* lg_exp_operand_next(Exp* exp) { return exp->operand_next; }
+link_public_api(const char *)
+	lg_exp_get_string(const Exp*);
+link_public_api(const char *)
+	lg_exp_stringify(const Exp *); /* Result is in static variable. */
 
 /**
  * The dictionary is stored as a binary tree comprised of the following
